@@ -20,8 +20,9 @@ import (
 	"errors"
 	"strings"
 
+	"github.com/go-kit/kit/log"
+	"github.com/go-kit/kit/log/level"
 	"github.com/prometheus/client_golang/prometheus"
-	"github.com/prometheus/common/log"
 )
 
 var errZFSNotAvailable = errors.New("ZFS / ZFS statistics are not available")
@@ -33,16 +34,21 @@ func init() {
 }
 
 type zfsCollector struct {
-	linuxProcpathBase string
-	linuxZpoolIoPath  string
-	linuxPathMap      map[string]string
+	linuxProcpathBase    string
+	linuxZpoolIoPath     string
+	linuxZpoolObjsetPath string
+	linuxZpoolStatePath  string
+	linuxPathMap         map[string]string
+	logger               log.Logger
 }
 
 // NewZFSCollector returns a new Collector exposing ZFS statistics.
-func NewZFSCollector() (Collector, error) {
+func NewZFSCollector(logger log.Logger) (Collector, error) {
 	return &zfsCollector{
-		linuxProcpathBase: "spl/kstat/zfs",
-		linuxZpoolIoPath:  "/*/io",
+		linuxProcpathBase:    "spl/kstat/zfs",
+		linuxZpoolIoPath:     "/*/io",
+		linuxZpoolObjsetPath: "/*/objset-*",
+		linuxZpoolStatePath:  "/*/state",
 		linuxPathMap: map[string]string{
 			"zfs_abd":         "abdstats",
 			"zfs_arc":         "arcstats",
@@ -56,14 +62,23 @@ func NewZFSCollector() (Collector, error) {
 			"zfs_zfetch":      "zfetchstats",
 			"zfs_zil":         "zil",
 		},
+		logger: logger,
 	}, nil
 }
 
 func (c *zfsCollector) Update(ch chan<- prometheus.Metric) error {
+
+	if _, err := c.openProcFile(c.linuxProcpathBase); err != nil {
+		if err == errZFSNotAvailable {
+			level.Debug(c.logger).Log("err", err)
+			return ErrNoData
+		}
+	}
+
 	for subsystem := range c.linuxPathMap {
 		if err := c.updateZfsStats(subsystem, ch); err != nil {
 			if err == errZFSNotAvailable {
-				log.Debug(err)
+				level.Debug(c.logger).Log("err", err)
 				// ZFS /proc files are added as new features to ZFS arrive, it is ok to continue
 				continue
 			}
@@ -110,3 +125,35 @@ func (c *zfsCollector) constPoolMetric(poolName string, sysctl zfsSysctl, value
 		poolName,
 	)
 }
+
+func (c *zfsCollector) constPoolObjsetMetric(poolName string, datasetName string, sysctl zfsSysctl, value uint64) prometheus.Metric {
+	metricName := sysctl.metricName()
+
+	return prometheus.MustNewConstMetric(
+		prometheus.NewDesc(
+			prometheus.BuildFQName(namespace, "zfs_zpool_dataset", metricName),
+			string(sysctl),
+			[]string{"zpool", "dataset"},
+			nil,
+		),
+		prometheus.UntypedValue,
+		float64(value),
+		poolName,
+		datasetName,
+	)
+}
+
+func (c *zfsCollector) constPoolStateMetric(poolName string, stateName string, isActive uint64) prometheus.Metric {
+	return prometheus.MustNewConstMetric(
+		prometheus.NewDesc(
+			prometheus.BuildFQName(namespace, "zfs_zpool", "state"),
+			"kstat.zfs.misc.state",
+			[]string{"zpool", "state"},
+			nil,
+		),
+		prometheus.GaugeValue,
+		float64(isActive),
+		poolName,
+		stateName,
+	)
+}
